<?php
namespace Teknisa\CADEST\Services;

use Doctrine\DBAL\DriverManager;
use Doctrine\Common\EventManager;
use Doctrine\ORM\EntityManager;

class ConnectionService {

    const RETURN_TYPE_DATA_SET = "dataset";
    const RETURN_TYPE_AFFECTED_ROWS = "affected_rows";
    const RETURN_TYPE_ERROR = "error";
    const RETURN_TYPE_MESSAGE = "message";
    const RETURN_TYPE_DATA_SET_ARRAY = "array";
    const RETURN_ISNT_TYPE_ALLOWED = "This query cannot be executed.";

    protected $connection;

    /**
     * Class constructor.
     *
     * @param $connectionParams all params to connect with database.
     */
    public function __construct(EntityManager $entityManager) {
        $this->connection = $entityManager->getConnection();
    }

    /**
     * Create a connection with database
     *
     * @param $user
     * @param $password
     *
     * @return $connection
     */
    public function dbConnection($user, $password) {

        $eventManager = new EventManager();

        $params["host"] = $this->host;
        $params["port"] = $this->port;
        $params["user"] = $user;
        $params["password"] = $password;
        $params["dbname"] = $this->dbName;
        $params["driver"] = $this->driver;
        $params["service"] = $this->service;
        $params["charset"] = $this->charset;

        if($params['driver'] === 'oci8') {
            $params['driverClass'] = '\Zeedhi\Framework\DBAL\Driver\OCI8\Driver';
            $eventManager->addEventSubscriber(new OracleSessionInit(array(
            'NLS_DATE_FORMAT' => "DD/MM/YYYY HH24:MI:SS",
            'NLS_LANGUAGE' => 'BRAZILIAN PORTUGUESE'
            )));
        }

        return DriverManager::getConnection($params, null, $eventManager);
    }

    /**
     * Find what type of SQL query is inputted by the user.
     * Will return one of the following "UPDATE", "INSERT", "DELETE" or "SELECT".
     *
     * @param string $query The user query.
     *
     * @return string
     */
    protected function getQueryType($query) {
        $queryType = explode(" ", $query);
        $queryType = strtoupper($queryType[0]);
        return $queryType;
    }

    /**
     * Execute a query in row to SELECT or REPLACE
     *
     * @param connection $connection
     * @param row $row
     *
     * @return array rows
     */
    protected function executeSelect($connection, $row){
        $statement = $connection->prepare($row['query']);
        $statement->execute();
        if(!isset($statement->errorCode)){
            $rows = array();

            while ($row = $statement->fetch()) {
                array_push($rows, $row);
            }

            $return = array(
                "type" => self::RETURN_TYPE_DATA_SET,
                "rows" => $rows
            );
        }else{
            $return = array(
                "type"  => self::RETURN_TYPE_ERROR,
                "error" => array(array("Error"=>$statement->errorInfo()))
            );
        }

        return $return;
    }

    /**
     * Execute a query in row to DELETE or UPDATE
     *
     * @param connection $connection
     * @param row $row
     *
     * @return array $retunr with the number of rows modified
     */
    protected function executeDelete($connection, $row){
        $result = $connection->query($row['query']);
        $countRows = $result->rowCount();
        return array(
            "type"  => self::RETURN_TYPE_AFFECTED_ROWS,
            "count" => array($countRows)
        );
    }

    /**
     * Execute a query in row to CREATE, DROP, ALTER or INSERT
     *
     * @param connection $connection
     * @param row $row
     *
     * @return array $return a query status after executed
     */
    protected function executeCreate($connection, $row){
        try{
            $result = $connection->query($row['query']);
            $return = array(
                "type"  => self::RETURN_TYPE_MESSAGE,
                "message" => array(array("Message"=>'Query executed successfully!'))
            );
        } catch (Exception $e) {
            $return = array(
                "type"  => self::RETURN_TYPE_ERROR,
                "error" => array(array("Error"=>$e->getMessage()))
            );
        }
        return $return;
    }

    /**
     * Execute a query in row to default mode
     *
     * @param connection $connection
     * @param row $row
     *
     * @return array $return a array with rows
     */
    protected function executeDefault($connection, $row){
        $rows = array();
        $stmt = $connection->query($row['query']);
        while ($row = $stmt->fetch()) {
            $rows[] =  $row;
        }
        return array(
            "type"  => self::RETURN_TYPE_DATA_SET,
            "rows" => $rows
        );
    }

    public function executeQueryInConnection($row) {

        $connection = $this->connection;
        $queryType = $this->getQueryType($row["query"]);
        $return = [self::RETURN_ISNT_TYPE_ALLOWED];

        switch ($queryType) {
            case "SELECT":
            case "REPLACE":
                $return = $this->executeSelect($connection, $row);
                break;
            case "UPDATE":
            case "DELETE":
                $return = $this->executeDelete($connection, $row);
                break;
            case "CREATE":
            case "DROP":
            case "ALTER":
            case "INSERT":
                $return = $this->executeCreate($connection, $row);
                break;
            default:
                $return = $this->executeDefault($connection, $row);
                break;
        }

        return $return;
    }

    public function executeQueryArrayInConnection($row) {
        $querys = $row['query'];

        $response = array();
        foreach($querys as $query){
            $row['query'] = $query;
            array_push($response,$this->executeQueryInConnection($row));
        }

        $return = array(
            "type"  => self::RETURN_TYPE_DATA_SET_ARRAY,
            "logs" => $response
        );
        return $return;
    }
}